use std::convert::TryFrom;
use super::errors;

pub struct XMASDecoder {
    input: Vec<i64>,
}

impl TryFrom<&str> for XMASDecoder {
    type Error = errors::Errors;

    fn try_from(input: &str) -> Result<Self, Self::Error> {
        let input = input
            .split_ascii_whitespace()
            .map(|line| line.trim().parse::<i64>())
            .collect::<std::result::Result<Vec<i64>, _>>()?;
        Ok(Self { input })
    }
}

impl XMASDecoder {
    pub fn find_continues(&self, target: i64) -> Option<&[i64]> {
        for offset in 0..self.input.len() {
            let slice = &self.input[offset..self.input.len()];
            if let Some(found) = Self::find_sum_slice(slice, target) {
                return Some(found);
            }
        }
        None
    }

    fn find_sum_slice(slice: &[i64], target: i64) -> Option<&[i64]> {
        let mut offset = 0;
        let mut sum = 0;
        for i in slice {
            offset += 1;
            sum += *i;
            if sum == target {
                return Some(&slice[0..offset]);
            } else if sum > target {
                return None;
            }
        }
        None
    }

    pub fn decode(&self) -> Option<i64> {
        self.input
            .windows(26)
            .into_iter()
            .filter_map(Self::validate_slice)
            .next()
    }

    fn validate_slice(window: &[i64]) -> Option<i64> {
        let (target, slice) = window.split_last().unwrap();
        for (offset, a) in slice.iter().enumerate() {
            for b in slice[offset + 1..slice.len()].iter() {
                if a + b == *target {
                    return None;
                }
            }
        }
        Some(*target)
    }
}
